/*
 * Copyright (C) 2014-2019 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __FA_BOARD_H__
#define __FA_BOARD_H__

#include "fc6-constants.h"
#include "FipCode6/fipcode6.h"

#if defined(__linux__)
#   include <asm/types.h>
#elif defined(__APPLE__) || defined(__QNX__)
#   define __u8     uint8_t
#   define __s16    int16_t
#   define __u16    uint16_t
#   define __s32    int32_t
#   define __u32    uint32_t
#   define __s64    int64_t
#   define __u64    uint64_t
#elif defined(_WIN32) || defined(_WIN64)
#   ifdef FD_WINDOWS_DRIVER
#       include <ntddk.h>
#   else
#       include <windows.h>
#       include <winioctl.h>
#   endif
#   define __u8     UCHAR
#   define __s16    SHORT
#   define __u16    USHORT
#   define __s32    LONG
#   define __u32    ULONG
#   define __s64    LONGLONG
#   define __u64    ULONGLONG
//BR
//HKEY_LOCAL_MACHINE/SYSTEM/ControlSet001/Services/PciFipArb/Parameters/Active_Devices
//HKEY_LOCAL_MACHINE/SYSTEM/ControlSet001/Services/PciFipArb/Parameters/Active_Indexes
//HKEY_LOCAL_MACHINE/SYSTEM/ControlSet001/Services/PciFipArb/Parameters/Used_Devices
//HKEY_LOCAL_MACHINE/SYSTEM/ControlSet001/Services/PciFipArb/Parameters/Used_Indexes
#   define REGISTRY_SUBKEY_PARAMETERS_NAME         TEXT("\\Parameters")
#   define REGISTRY_VALUE_NBOFACTIVEDEVICES_NAME   TEXT("Active_Devices")
#   define REGISTRY_VALUE_ACTIVEINDEX_NAME         TEXT("Active_Indexes")
#   define REGISTRY_VALUE_NBOFUSEDDEVICES_NAME     TEXT("Used_Devices")
#   define REGISTRY_VALUE_USEDINDEX_NAME           TEXT("Used_Indexes")
#endif

#if defined(__QNX__)
#   include <devctl.h>
#endif

#define DRV_NAME "pcifip"
#define DRV_HDR "pcifip: "

// Maximum number of cards of the same type that can be supported
// (Limitation software to 4 in user mode)
#define MAX_DEVICE              9

// Maximum size of the usable area of ​​the EEPROM in 16-bit words
#define EEPROM_USABLE_SIZE      0x60 // 96 mots

// Masques utilisés pour l'IOCTL IOCTL_EDA_CHECK_EEPROM
#define C_MASK_EEPROM_PRESENT   0x00000001
#define C_MASK_EEPROM_NOT_BLANK 0x00000002

#define PLX_REGISTERS_TABLE_SIZE    21

// Device ID
// For backward compatibility
// But this is not normal. Values ​​should be:
// PLX 9052: Vendor ID: 0x10b5 (PLX) - Device ID: 0x9052
// PLX 9030: Vendor ID: 0x10b5 (PLX) - Device ID: 0x9030
#define PCI_VEN_ID_PLX_9030         0x10b5
#define PCI_DEV_ID_PLX_9030         0x5201 

// Définition des registres PCI9030 (PLX Technology) pour les cartes FIP.
// (multiple de 4 en raison de la gestion du PLX - Bus local 32-bit)
#define C_PLX_REGISTER_NUMBER   21

//**** PCI CONFIG REGISTERS
#define C_REG_PLX_CS3BASE       0x48
#define C_REG_PLX_INTCSR        0x4C
#define C_REG_PLX_CNTRL         0x50
#define C_REG_PLX_GPIOC         0x54    // LEDs ctrl
// => Relative to Exoligent
#define C_REG_PLX_RESOURCES     0x58
#define C_REG_PLX_CTRL          0x5C
#define C_REG_PLX_CONFIG        0x60
#define C_REG_PLX_TEMPERATURE   0x64
#define C_REG_PLX_ID1           0x68
#define C_REG_PLX_ID2           0x6C
#define C_REG_PLX_PMDATASEL     0x70
#define C_REG_PLX_PMDATASCALE   0x74

// Defining useful masks
// --- on INTCSR
#define C_MK_REG_BIT_IT_LINT1_ENABLE            (1 << 0)
#define C_MK_REG_BIT_IT_LINT1_POLARITY_1        (1 << 1)
#define C_MK_REG_BIT_IT_LINT1_ACTIVE            (1 << 2)
#define C_MK_REG_BIT_IT_LINT2_ENABLE            (1 << 3)
#define C_MK_REG_BIT_IT_LINT2_POLARITY_1        (1 << 4)
#define C_MK_REG_BIT_IT_LINT2_ACTIVE            (1 << 5)
#define C_MK_REG_BIT_IT_ENABLE                  (1 << 6)
#define C_MK_REG_BIT_SOFT_ENABLE                (1 << 7)
#define C_MK_REG_BIT_IT_LINT1_EDGE_ENABLE       (1 << 8)
#define C_MK_REG_BIT_IT_LINT2_EDGE_ENABLE       (1 << 9)
#define C_MK_REG_BIT_IT_LINT1_EDGE_CLEAR        (1 << 10)
#define C_MK_REG_BIT_IT_LINT2_EDGE_CLEAR        (1 << 11)
// --- on CNTRL
#define C_MK_REG_BIT_EEPROM_CLOCK_HIGH          (1 << 24)
#define C_MK_REG_BIT_EEPROM_CS_HIGH             (1 << 25)
#define C_MK_REG_BIT_EEPROM_WRITE_HIGH          (1 << 26)
#define C_MK_REG_BIT_EEPROM_READ_BIT            (1 << 27)
#define C_MK_REG_BIT_EEPROM_PRESENT_HIGH        (1 << 28)
#define C_MK_REG_BIT_EEPROM_RELOAD_HIGH         (1 << 29)
#define C_MK_REG_BIT_LOCAL_RESET_ON             (1 << 30)
// --- on GPIOC
#define C_MK_REG_BIT_GPIO4_LED_ON               (1 << 14)
#define C_MK_REG_BIT_BLUE_LED_ON                (1 << 17)
#define C_MK_REG_BIT_GREEN_LED_ON               (1 << 20)
#define C_MK_REG_BIT_RED_LED_ON                 (1 << 23)
// --- on CARDCONF
#define C_MK_REG_PLX_TYPE                       (0x03 << 16)
#define C_MK_REG_CARD_SPEED                     (0x0D << 0)
#define C_MK_REG_BUS_IMP                        (0x03 << 4)
#define C_MK_REG_CARD_VERSION                   (0xFF << 8)
// --- on RESOURCES
#define C_MK_REG_FULLFIP_NB                     (0x07 << 0)
#define C_MK_REG_FIELDUAL_NB                    (0x07 << 4)
#define C_MK_REG_FULLFIP_RAM_NB                 (0x07 << 8)
#define C_MK_REG_PCIE_TYPE                      (0x03 << 14)
#define C_MK_REG_MAGIC_NB                       (0xFF << 24)
// --- on CTRL
#define C_MK_REG_INT                            0x7FF
#define C_MK_REG_FFA_RESET_ON                   (1 << 16)
#define C_MK_REG_FFB_RESET_ON                   (1 << 17)
#define C_MK_REG_SYNC                           (0x3F << 24)
//     -> INT
#define C_MK_REG_INT_FFA_IRQ_FLAG               (1 << 0)
#define C_MK_REG_INT_FFA_EOC_FLAG               (1 << 1)
#define C_MK_REG_INT_FFB_IRQ_FLAG               (1 << 2)
#define C_MK_REG_INT_FFB_EOC_FLAG               (1 << 3)
#define C_MK_REG_INT_SYN_EXT_FLAG               (1 << 4)
#define C_MK_REG_INT_FFA_EOC_POLARITY           (1 << 5)
#define C_MK_REG_INT_FFB_EOC_POLARITY           (1 << 6)
#define C_MK_REG_INT_FFA_IRQ_ENABLE             (1 << 7)
#define C_MK_REG_INT_FFA_EOC_ENABLE             (1 << 8)
#define C_MK_REG_INT_FFB_IRQ_ENABLE             (1 << 9)
#define C_MK_REG_INT_FFB_EOC_ENABLE             (1 << 10)
//     -> SYNC
#define C_MK_REG_SYNC_OEn                       (1 << 24)
#define C_MK_REG_SYNC_DIR                       (1 << 25)
#define C_MK_REG_SYNC_TST                       (1 << 26)
#define C_MK_REG_SYNC_50_OHM                    (1 << 27)
#define C_MK_REG_SYNC_INT_EXT_ENABLE            (1 << 28)
#define C_MK_REG_SYNC_INT_EXT_POLARITY          (1 << 29)
// --- on CONFIG
#define C_MK_REG_SPEED                          (0x07 << 0)
#define C_MK_REG_PG                             (0x0F << 8)
#define C_MK_REG_OPT                            (0x0F << 16)
#define C_MK_REG_GA                             (0x1F << 24)
//     -> PG - Power Good (Indicator)
#define C_MK_REG_PG_1V8_FLAG                    (1 << 8)
#define C_MK_REG_PG_2V5_FLAG                    (1 << 9)
#define C_MK_REG_PG_5V_FLAG                     (1 << 10)
#define C_MK_REG_PG_12V_FLAG                    (1 << 11)
//     -> OPT - Microswitches states
#define C_MK_REG_OPT_SW1_ON                     (1 << 16)
#define C_MK_REG_OPT_SW2_ON                     (1 << 17)
#define C_MK_REG_OPT_SW3_ON                     (1 << 18)
#define C_MK_REG_OPT_SW4_ON                     (1 << 19)
// --- on TEMPERATURE
#define C_MK_REG_TEMP_VALUE                     0xFFFF
#define C_MK_REG_TEMP_R0                        (1 << 21)
#define C_MK_REG_TEMP_R1                        (1 << 22)

// Defining useful values
// --- on RESOURCES
#define C_REG_MAGIC_NB                          0x57
// --- on CARDCONF
#define C_REG_PLX9052_TYPE                      0x00000000
#define C_REG_PLX9030_TYPE                      0x00010000
#define C_REG_SPEED_31K25                       0x00000002
#define C_REG_SPEED_1M                          0x00000004
#define C_REG_SPEED_2M5                         0x00000006
#define C_MK_REG_IMP_150                        0x00000000
#define C_MK_REG_IMP_120                        0x00000010

// Definition of PCI configuration registers for FIP cards.
// (multiple of 4 due to PLX management - 32-bit local bus)
#define C_REG_CNF_PCI_DEVID     0x02
#define C_REG_CNF_PCI_VNDID     0x00
#define C_REG_CNF_PCI_SUBSYS    0x2E
#define C_REG_CNF_PCI_SUBVND    0x2C

// Default value - EEPROM not valid
#define C_SUB_VENDOR_ID_DEFAULT_VALUE   0
#define C_SUB_SYSTEM_ID_DEFAULT_VALUE   0 

#define PCI_BOARD_INFO          "PCI FIP Arbiter"
#define PCIE_BOARD_INFO         "PCIe FIP Arbiter"
#define PCI104_BOARD_INFO       "PCI-104 FIP Arbiter"
#define PCIE104_BOARD_INFO      "PCIe/104 FIP Arbiter"
#define PXI_BOARD_INFO          "PXI FIP Arbiter"
#define PXIE_BOARD_INFO         "PXIe FIP Arbiter"
#define MPCIE_BOARD_INFO        "mPCIe FIP Arbiter"
#define CPCI_BOARD_INFO         "cPCI FIP Arbiter"

#define PCI_OPT_BOARD_INFO      "PCI Optical FIP Arbiter"
#define PCIE_OPT_BOARD_INFO     "PCIe Optical FIP Arbiter"
#define PCI104_OPT_BOARD_INFO   "PCI-104 Optical FIP Arbiter"
#define PCIE104_OPT_BOARD_INFO  "PCIe/104 Optical FIP Arbiter"
#define PXI_OPT_BOARD_INFO      "PXI Optical FIP Arbiter"
#define PXIE_OPT_BOARD_INFO     "PXIe Optical FIP Arbiter"
#define MPCIE_OPT_BOARD_INFO    "mPCIe Optical FIP Arbiter"
#define CPCI_OPT_BOARD_INFO     "cPCI Optical FIP Arbiter"

#define MAX_FA_BOARD 4

struct arg_io
{
    void* argIO_In;
    void* argIO_Out;
    __s32 lgIn, lgOut;
    __s32 io_status;
};

#pragma pack(1) // 1 byte alignment
//***************************************************************************************
//
//
//
//  EXCHANGE STRUCTURES BETWEEN "KERNEL MODE" (Driver) AND "USER MODE"
//
//
//
//***************************************************************************************
//
//
// INPUT STRUCTURES OF KERNEL MODE
//
//
//***************************************************************************************
// EDA_WRITE_REGISTERS_INPUT
// Input structure for
// IOCTL_EDA_PLX_WRITE_REGISTER calls
//***************************************************************************************
typedef struct  _EDA_WRITE_REGISTER_INPUT {
    __u32 RelativeAddress;
    __u32 Data;
} EDA_WRITE_REGISTER_INPUT;

//***************************************************************************************
// EDA_READ_EEPROM_INPUT
// Input structure for IOCTL_EDA_READ_EEPROM call
//***************************************************************************************
typedef struct _EDA_READ_EEPROM_INPUT {
    __u8 CountOfWords;                  // Number of words to read (16-bit)
    __u8 StartAddress;                  // Relative read-start address
} EDA_READ_EEPROM_INPUT;                // (pas 16-bit)

//***************************************************************************************
// EDA_WRITE_EEPROM_INPUT
// Input structure for IOCTL_EDA_WRITE_EEPROM call
//***************************************************************************************
typedef struct _EDA_WRITE_EEPROM_INPUT {
    __u8 CountOfWords;                  // Number of words to write (16-bit)
    __u8 StartAddress;                  // Relative write start address (pas 16-bit)
    __u8 Data[EEPROM_USABLE_SIZE * 2];  // Data table to be written (in bytes)
} EDA_WRITE_EEPROM_INPUT;

//***************************************************************************************
// FIP_WRITE_PHY_INPUT
// Input structure for IOCTL CMDE_W_PHY call
//***************************************************************************************
typedef struct _FIP_WRITE_PHY_INPUT {
    __u16 PageNumber;                   // Page number to write
    __u8  Data[NB_BYTE_PER_SEGMENT];    // Data to be written
} FIP_WRITE_PHY_INPUT;

//***************************************************************************************
// FIP_WRITE_WORD_INPUT
// Input structure for IOCTL CMDE_W_WORD call
//***************************************************************************************
typedef struct _FIP_WRITE_WORD_INPUT {
    __u32 Address;                      // Absolute address (20-bit)
    __u16 Data;                         // Data to be written (16-bit)
} FIP_WRITE_WORD_INPUT;

//***************************************************************************************
// FIP_WRITE_DESC_INPUT
// Input structure for IOCTL CMDE_W_DESC call
//***************************************************************************************
typedef struct _FIP_WRITE_DESC_INPUT {
    __u32 Address;                       // Absolute address (20-bit)
    __u16 Data[DESCRIPTOR_SIZE_IN_WORD]; // Data to be written (16-bit) - 16 word table
} FIP_WRITE_DESC_INPUT;

//***************************************************************************************
// FIP_WRITE_VAR_INPUT
// Input structure for IOCTL CMDE_W_VAR call
// Data ::=     {<Data_Var>}^n
//          ou  {<Data_Var>}^n<Production_Status>
//          ou  {<Data_Var>}^n<Production_Delay><Production_Status>
// Note:    n = [1 ... 126]: Variable without <Production_Status>
//          n = [1 ... 125]: Variable with <Production_Status>
//          n = [6 ... 120]: Variable with dynamic refresh
//***************************************************************************************
typedef struct _FIP_WRITE_VAR_INPUT {
    __u16 AccessKey;
    __u8  Data[NB_BYTE_PER_SEGMENT - 2];
} FIP_WRITE_VAR_INPUT;

//***************************************************************************************
// FIP_SEND_APER_INPUT
// Input structure for IOCTL CMDE_SEND_APER call
//***************************************************************************************
typedef struct  _FIP_SEND_APER_INPUT {
    __u16 AccessKey;                
    __u8  TypeCmd;  // CMDE_SEND_APU or CMDE_SEND_APN
} FIP_SEND_APER_INPUT;

//***************************************************************************************
// FIP_WRITE_MSG_INPUT
// Input structure for IOCTL CMDE_W_MSG call
//***************************************************************************************
typedef struct  _FIP_WRITE_MSG_INPUT {
    // Channel number
    __u16  ChannelNumber;
    // Message characteristics          
    __u8  MsgMode;
    __u8  ReservedByte;
    __u8  MsgLengthMsb; // Headers + useful data
    __u8  MsgLengthLsb;
    // Message header
    __u8  DestIdMsb;
    __u8  DestIdLsb;
    __u8  DestIdSegment;
    __u8  SourceIdMsb;
    __u8  SourceIdLsb;
    __u8  SourceIdSegment;
    // Useful data
    __u8  Data[MAX_MSG_LENGTH_WITHOUT_CHARACT_AND_HEADER];
} FIP_WRITE_MSG_INPUT;

//***************************************************************************************
// FIP_CHANGE_PARAM_INPUT
// Input structure for IOCTL CMDE_CHANGE_PARAM call
//***************************************************************************************
typedef struct _FIP_CHANGE_PARAM_INPUT {
    __u16 BA_Subscriber_Number;
    __u16 BA_Max_Subscriber_Number;
    __u16 BA_Priority;
    __u16 BA_Start_Time;
    __u16 BA_Election_Time;
} FIP_CHANGE_PARAM_INPUT;

//
//
// OUTPUT STRUCTURES OF KERNEL MODE
//
//
//***************************************************************************************
// EDA_DEV_INFO_OUTPUT
// Output structure from IOCTL_EDA_GET_DEV_INFO call
//***************************************************************************************
typedef struct  _EDA_DEV_INFO_OUTPUT {
    /* device general info */
    __u16           vid;
    __u16           did;
    __u16           ssvid;
    __u16           ssid;
    /* physical BAR addresses */
    unsigned long   plx_base;
    unsigned long   plx_size;
    int             nb_fullfip;
    unsigned long   fullfip_base;
    unsigned long   fullfip_size;
    int             nb_fullfip_ram;
    unsigned long   fullfip_ram_base;
    unsigned long   fullfip_ram_size;
    int             nb_fieldual;
    unsigned long   fieldual_base;
    unsigned long   fieldual_size;
    /* irq infos*/
    int             irq_number;
    /* fullfip/fieldual offset factor for the registers */
    int             regs_offset_factor;
} EDA_DEV_INFO_OUTPUT;

//***************************************************************************************
// EDA_LIST_INTERNAL_REGISTERS_OUTPUT
// Output structure from IOCTL_EDA_LIST_INTERNAL_REGISTERS call
//***************************************************************************************
typedef struct  _EDA_LIST_INTERNAL_REGISTERS_OUTPUT {
    __u32  Register[21];
} EDA_LIST_INTERNAL_REGISTERS_OUTPUT;

//***************************************************************************************
// EDA_READ_EEPROM_OUTPUT
// Output structure from IOCTL_EDA_READ_EEPROM call
//***************************************************************************************
typedef struct _EDA_READ_EEPROM_OUTPUT {
    __u8 CountOfBytes;                  // Number of bytes read
    __u8 Data[EEPROM_USABLE_SIZE * 2];  // Data read
} EDA_READ_EEPROM_OUTPUT;

//***************************************************************************************
// EDA_GET_STATS_OUTPUT
// Output structure from IOCTL_EDA_GET_STATS call
//***************************************************************************************
typedef struct _EDA_GET_STATS_OUTPUT {
    /*** Hardware IRQ handled */ 
    __u32 IrqHandledCount;
    __u32 IrqNonHandledCount;

    /*** EOC */
    // Number of macrocycles - linked with fip synchro var (0x9003)
    __u64 CycleCount;
    // Macrocycle perdiod - nanosecond resolution
    __u64 CycleTime;

    /*** IRQ */
    // Number of variables sent with a network event flag sets to "permanent"
    __u64 VarPSentCount;
    // Number of variables sent with a network event flag sets to "temporary"
    __u64 VarTSentCount;
    // Number of variables received with a network event flag sets to "permanent"
    __u64 VarPReceivedCount;
    // Number of variables received with a network event flag sets to "temporary"
    __u64 VarTReceivedCount;
    // Number of message received
    __u64 MsgReceivedCount;
    // Number of message sent
    __u64 MsgSentCount;
    // Number of "urgent" requests for aperiodic variable list sent
    __u64 APUListSentCount;
    // Number of "normal" requests for aperiodic variable list sent
    __u64 APNListSentCount;
    // Counter of bus arbiter (re)activation
    __u64 BaActivityCount;
    // Counter of stopping of bus arbiter following timeout
    __u64 BaStopOnTimeoutCount;
    // Counter of stopping of bus arbiter following network anomalies
    __u64 BaStopOnAnomaliesCount;
    // Counter of stopping of bus arbiter following user request
    __u64 BaStopByUserCount;
    // Counter of return of bus arbiter to idle status
    __u64 BaIdleCount;
    // Counter of bus arbiter suspend
    __u64 BaSuspendCount;

} EDA_GET_STATS_OUTPUT;

//***************************************************************************************
// EDA_GET_MONITORING_INFOS_OUTPUT
// Output structure from IOCTL_EDA_GET_MONITORING_INFOS call
//***************************************************************************************
typedef struct _EDA_GET_MONITORING_INFOS_OUTPUT {
    __s16 board_temperature; // 2's complement (16-bit signed)

} EDA_GET_MONITORING_INFOS_OUTPUT;

//***************************************************************************************
// FIP_VALID_CIRCUIT_OUTPUT
// Output structure from IOCTL CMDE_VALID call
//***************************************************************************************
typedef struct  _FIP_VALID_CIRCUIT_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;         // <Var_State_Cr>
    __u8  Uflags;
} FIP_VALID_CIRCUIT_OUTPUT;

//***************************************************************************************
// FIP_GET_MEDIUM_OUTPUT
// Output structure from IOCTL CMDE_GET_MEDIUM call
//***************************************************************************************
typedef struct _FIP_GET_MEDIUM_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                         // <Var_State_Cr>
    __u8  Data[FIP_NETWORK_STATUS_IN_BYTE]; // Data read - Network status table
} FIP_GET_MEDIUM_OUTPUT;

//***************************************************************************************
// FIP_READ_PHY_OUTPUT
// Output structure from IOCTL CMDE_R_PHY call
//***************************************************************************************
typedef struct  _FIP_READ_PHY_OUTPUT {
   // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Cr>
    __u8  Data[NB_BYTE_PER_SEGMENT];    // Data read
} FIP_READ_PHY_OUTPUT;

//***************************************************************************************
// FIP_READ_WORD_OUTPUT
// Output structure from IOCTL CMDE_R_PHY call
//***************************************************************************************
typedef struct _FIP_READ_WORD_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Cr>
    __u16 Data;                         // Data read
} FIP_READ_WORD_OUTPUT;

//***************************************************************************************
// FIP_READ_VAR_OUTPUT
// Output structure from IOCTL CMDE_R_VAR call
// Size16 <=> <Length>
// Data ::= <Pdu><Length>{<Data_Var>}^Length-2
//          ou  <Pdu><Length>{<Data_Var>}^Length-3<Production_Status>
// Note: <Data_Var> max length = 126 octets
// VarState ::= <Var_State_Var>
//***************************************************************************************
typedef struct _FIP_READ_VAR_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Var>
    // VAR frame size
    __u16 Size16;
    // VAR frame read
    __u8  Data[NB_BYTE_PER_SEGMENT];
} FIP_READ_VAR_OUTPUT;

//***************************************************************************************
// FIP_READ_VAR_TIME_OUTPUT
// Output structure from IOCTL CMDE_R_VAR_TIME call
// Size16 <=> <Length>
// Data ::= <Pdu><Length>{<Data_Var>}^Length-7<Production_Delay><Production_Status>
// Note: <Data_Var> max length = 121 octets
// VarState ::= <Var_State_Var>
//***************************************************************************************
typedef struct _FIP_READ_VAR_TIME_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Var>
    // VAR_TIME frame size
    __u16 Size16;
    __u16 ConsumptionTime;
    // VAR_TIME frame read
    __u8  Data[NB_BYTE_PER_SEGMENT];
} FIP_READ_VAR_TIME_OUTPUT;

//***************************************************************************************
// FIP_READ_LSTPRESENT_OUTPUT
// Output structure from IOCTL CMDE_R_LPRESENT call
//***************************************************************************************
typedef struct _FIP_READ_LSTPRESENT_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                 // <Var_State_Var>
    // Tables of presence
    __u8  Pdu;
    __u8  TotalLength;
    __u8  MediumTab1Id;
    __u8  MediumTab1Length; 
    __u8  MediumTab1[32];
    __u8  MediumTab2Id;
    __u8  MediumTab2Length;
    __u8  MediumTab2[32];
} FIP_READ_LSTPRESENT_OUTPUT;

//***************************************************************************************
// FIP_READ_EVT_OUTPUT
// Output structure from IOCTL CMDE_R_EVT call
//***************************************************************************************
typedef struct  _FIP_READ_EVT_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Cr>
    // Received event code
    __u8  EventCodeMsb;
    __u8  EventCodeLsb;
    __u8  EventParameterMsb;
    __u8  EventParameterLsb;
} FIP_READ_EVT_OUTPUT;

//***************************************************************************************
// FIP_READ_MSG_OUTPUT
// Output structure from IOCTL CMDE_R_MSG call
//***************************************************************************************
typedef struct  _FIP_READ_MSG_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Msg>
    // Characteristics of the message
    __u16 MsgReceptionTime;
    __u16 MsgLength;
    // Message header
    __u8  DestIdMsb;
    __u8  DestIdLsb;
    __u8  DestIdSegment;
    __u8  SourceIdMsb;
    __u8  SourceIdLsb;
    __u8  SourceIdSegment;
    // Useful message data
    __u8  Data[MESSAGING_SEGMENT_TABLE_SIZE];
} FIP_READ_MSG_OUTPUT;

//***************************************************************************************
// FIP_PURGE_MSG_OUTPUT
// Output structure from IOCTL CMDE_PURGE_MSG call
//***************************************************************************************
typedef struct _FIP_PURGE_MSG_OUTPUT {
    // Execution report provided by the driver
    __u8  VarState;                     // <Var_State_Cr>
    // Number of blocks released
    __u16 BlockNumber;                  // Size16 / 2
    // Table containing the released blocks
    __u16 BlockFreed[NB_MAX_FIFO_EMISSION_BLOCK];
} FIP_PURGE_MSG_OUTPUT;

//***************************************************************************************
//
//
//
//  EXCHANGE STRUCTURES FOR MOBIFIP (HOST <-> MobiFip)
//
//
//
//***************************************************************************************
typedef struct stFullFipInRq {
    __u8 key_l; 
    __u8 key_h;
    __u8 data[510];
} FullFipInRq;

typedef struct stFullFipOutRq {
    __u8 ustate;
    __u8 varstate;
    __u8 data[510];
} FullFipOutRq;

typedef struct stFielDualInRq {
    __u8 command;
    __u8 reserved;
    __u8 data[510];
} FielDualInRq;

typedef struct stFielDualOutRq {
    __u8 reserved;
    __u8 channel_status;
    __u8 data[510];
} FielDualOutRq;
#pragma pack()

//***************************************************************************************
//                              IOCTLs Definitions
//***************************************************************************************
#if defined(_WIN32) || defined(_WIN64)

#define C_FIP_TYPE 40000

// IOCTL EDA (ExoDevice Access)
// --- Management
#define IOCTL_EDA_GET_DEV_INFO \
    CTL_CODE(C_FIP_TYPE, 0x801, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_LIST_INTERNAL_REGISTERS \
    CTL_CODE(C_FIP_TYPE, 0x803, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_RELOAD_INTERNAL_REGISTERS \
    CTL_CODE(C_FIP_TYPE, 0x814, METHOD_BUFFERED, FILE_WRITE_ACCESS)
#define IOCTL_EDA_PLX_WRITE_REGISTER \
    CTL_CODE(C_FIP_TYPE, 0x806, METHOD_BUFFERED, FILE_WRITE_ACCESS)
#define IOCTL_EDA_PLX_READ_REGISTER \
    CTL_CODE(C_FIP_TYPE, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- EEPROM Management
#define IOCTL_EDA_CHECK_EEPROM \
    CTL_CODE(C_FIP_TYPE, 0x802, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_READ_EEPROM \
    CTL_CODE(C_FIP_TYPE, 0x80C, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_WRITE_EEPROM \
    CTL_CODE(C_FIP_TYPE, 0x80D, METHOD_BUFFERED, FILE_WRITE_ACCESS)

// --- Useful I/O access
#define IOCTL_EDA_GET_RGB_LED \
    CTL_CODE(C_FIP_TYPE, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_SET_RGB_LED \
    CTL_CODE(C_FIP_TYPE, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Useful information access
#define IOCTL_EDA_GET_PRODUCT_NUMBER \
    CTL_CODE(C_FIP_TYPE, 0x80E, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_SET_PRODUCT_NUMBER \
    CTL_CODE(C_FIP_TYPE, 0x80A, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_DEVICE_ID \
    CTL_CODE(C_FIP_TYPE, 0x80B, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_SET_DEVICE_ID \
    CTL_CODE(C_FIP_TYPE, 0x810, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_CARD_CONF \
    CTL_CODE(C_FIP_TYPE, 0x80F, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_EDA_SET_CARD_CONF \
    CTL_CODE(C_FIP_TYPE, 0x811, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_SUBSYSTEM_VENDOR_ID \
    CTL_CODE(C_FIP_TYPE, 0x815, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_SET_SUBSYSTEM_VENDOR_ID \
    CTL_CODE(C_FIP_TYPE, 0x816, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_SUBSYSTEM_ID \
    CTL_CODE(C_FIP_TYPE, 0x817, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_SET_SUBSYSTEM_ID \
    CTL_CODE(C_FIP_TYPE, 0x818, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_REVISION_ID \
    CTL_CODE(C_FIP_TYPE, 0x819, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_SET_REVISION_ID \
    CTL_CODE(C_FIP_TYPE, 0x81A, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_MONITORING_INFOS \
    CTL_CODE(C_FIP_TYPE, 0x81B, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- FullFip access
#define IOCTL_EDA_FULLFIP_RESET \
    CTL_CODE(C_FIP_TYPE, 0x807, METHOD_BUFFERED, FILE_WRITE_ACCESS)

// --- FielDual / MAX3000A access
#define IOCTL_EDA_FIELDUAL_VALID_MEDIUM \
    CTL_CODE(C_FIP_TYPE, 0x81E, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_FIELDUAL_GET_MEDIUM \
    CTL_CODE(C_FIP_TYPE, 0x81F, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- IRQ management
#define IOCTL_EDA_IRQ_WAIT_FOR \
    CTL_CODE(C_FIP_TYPE, 0x820, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_IRQ_REARM \
    CTL_CODE(C_FIP_TYPE, 0x821, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_IRQ_RESUME \
    CTL_CODE(C_FIP_TYPE, 0x822, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Statistics
#define IOCTL_EDA_RESET_STATS \
    CTL_CODE(C_FIP_TYPE, 0x823, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_EDA_GET_STATS \
    CTL_CODE(C_FIP_TYPE, 0x824, METHOD_BUFFERED, FILE_ANY_ACCESS)

// IOCTL FCA (FipCode Access) - related to FullFip IOSpace
// --- System Management
#define IOCTL_FCA_START \
    CTL_CODE(C_FIP_TYPE, 0x830, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_TST_WFIFO \
    CTL_CODE(C_FIP_TYPE, 0x831, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_TST_RFIFO \
    CTL_CODE(C_FIP_TYPE, 0x832, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_TST_IRQ \
    CTL_CODE(C_FIP_TYPE, 0x833, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_TST_REG \
    CTL_CODE(C_FIP_TYPE, 0x834, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_TST_TIMER \
    CTL_CODE(C_FIP_TYPE, 0x835, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_VALID \
    CTL_CODE(C_FIP_TYPE, 0x836, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_GET_MEDIUM \
    CTL_CODE(C_FIP_TYPE, 0x837, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_VALID_MEDIUM \
    CTL_CODE(C_FIP_TYPE, 0x838, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_NOISE_MANAGEMENT \
    CTL_CODE(C_FIP_TYPE, 0x839, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Memory Access
#define IOCTL_FCA_W_PHY \
    CTL_CODE(C_FIP_TYPE, 0x83A, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_PHY \
    CTL_CODE(C_FIP_TYPE, 0x83B, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_W_WORD \
    CTL_CODE(C_FIP_TYPE, 0x83C, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_WORD \
    CTL_CODE(C_FIP_TYPE, 0x83D, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_W_DESC \
    CTL_CODE(C_FIP_TYPE, 0x83E, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Variable Access
#define IOCTL_FCA_W_VAR \
    CTL_CODE(C_FIP_TYPE, 0x83F, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_VAR \
    CTL_CODE(C_FIP_TYPE, 0x840, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_VAR_SVF \
    CTL_CODE(C_FIP_TYPE, 0x841, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_VAR_TIME \
    CTL_CODE(C_FIP_TYPE, 0x842, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_LPRESENT \
    CTL_CODE(C_FIP_TYPE, 0x843, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_SEND_APER \
    CTL_CODE(C_FIP_TYPE, 0x844, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_PURGE_APER \
    CTL_CODE(C_FIP_TYPE, 0x845, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Event Access
#define IOCTL_FCA_R_EVT \
    CTL_CODE(C_FIP_TYPE, 0x846, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_ACQ_EOC \
    CTL_CODE(C_FIP_TYPE, 0x847, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Message Access
#define IOCTL_FCA_W_MSG \
    CTL_CODE(C_FIP_TYPE, 0x848, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_R_MSG \
    CTL_CODE(C_FIP_TYPE, 0x849, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_PURGE_MSG \
    CTL_CODE(C_FIP_TYPE, 0x84A, METHOD_BUFFERED, FILE_ANY_ACCESS)

// --- Ba Access
#define IOCTL_FCA_START_BA \
    CTL_CODE(C_FIP_TYPE, 0x84B, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_STOP_BA \
    CTL_CODE(C_FIP_TYPE, 0x84C, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_CONTINUE_BA \
    CTL_CODE(C_FIP_TYPE, 0x84D, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_CHANGE_MAC \
    CTL_CODE(C_FIP_TYPE, 0x84E, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_SYNC_BA \
    CTL_CODE(C_FIP_TYPE, 0x84F, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_FCA_CHANGE_PARAM \
    CTL_CODE(C_FIP_TYPE, 0x850, METHOD_BUFFERED, FILE_ANY_ACCESS)

#elif defined(__linux__) || defined(__APPLE__)

#define IOCTL_PARAM_SIZE 255 
#define MEMORY_DEVICE 0xFF 

// IOCTL EDA (ExoDevice Access)
// --- Management
#define IOCTL_EDA_GET_DEV_INFO              _IOWR(MEMORY_DEVICE, 0, int)
#define IOCTL_EDA_LIST_INTERNAL_REGISTERS   _IOWR(MEMORY_DEVICE, 1, int)
#define IOCTL_EDA_RELOAD_INTERNAL_REGISTERS _IOWR(MEMORY_DEVICE, 2, int)
#define IOCTL_EDA_PLX_WRITE_REGISTER        _IOWR(MEMORY_DEVICE, 3, int)
#define IOCTL_EDA_PLX_READ_REGISTER         _IOWR(MEMORY_DEVICE, 4, int)

// --- EEPROM Management
#define IOCTL_EDA_CHECK_EEPROM              _IOWR(MEMORY_DEVICE, 5, int)
#define IOCTL_EDA_READ_EEPROM               _IOWR(MEMORY_DEVICE, 6, int)
#define IOCTL_EDA_WRITE_EEPROM              _IOWR(MEMORY_DEVICE, 7, int)

// --- Useful I/O access
#define IOCTL_EDA_GET_RGB_LED               _IOWR(MEMORY_DEVICE, 8, int)
#define IOCTL_EDA_SET_RGB_LED               _IOWR(MEMORY_DEVICE, 9, int)

// --- Useful information access
#define IOCTL_EDA_GET_PRODUCT_NUMBER        _IOWR(MEMORY_DEVICE, 10, int)
#define IOCTL_EDA_SET_PRODUCT_NUMBER        _IOWR(MEMORY_DEVICE, 11, int)
#define IOCTL_EDA_GET_DEVICE_ID             _IOWR(MEMORY_DEVICE, 12, int)
#define IOCTL_EDA_SET_DEVICE_ID             _IOWR(MEMORY_DEVICE, 13, int)
#define IOCTL_EDA_GET_CARD_CONF             _IOWR(MEMORY_DEVICE, 14, int)
#define IOCTL_EDA_SET_CARD_CONF             _IOWR(MEMORY_DEVICE, 15, int)
#define IOCTL_EDA_GET_SUBSYSTEM_VENDOR_ID   _IOWR(MEMORY_DEVICE, 20, int)
#define IOCTL_EDA_SET_SUBSYSTEM_VENDOR_ID   _IOWR(MEMORY_DEVICE, 21, int)
#define IOCTL_EDA_GET_SUBSYSTEM_ID          _IOWR(MEMORY_DEVICE, 22, int)
#define IOCTL_EDA_SET_SUBSYSTEM_ID          _IOWR(MEMORY_DEVICE, 23, int)
#define IOCTL_EDA_GET_REVISION_ID           _IOWR(MEMORY_DEVICE, 24, int)
#define IOCTL_EDA_SET_REVISION_ID           _IOWR(MEMORY_DEVICE, 25, int)
#define IOCTL_EDA_GET_MONITORING_INFOS      _IOWR(MEMORY_DEVICE, 26, int)

// --- FullFip access
#define IOCTL_EDA_FULLFIP_RESET             _IOWR(MEMORY_DEVICE, 28, int)

// --- FielDual / MAX3000A access
#define IOCTL_EDA_FIELDUAL_VALID_MEDIUM     _IOWR(MEMORY_DEVICE, 29, int)
#define IOCTL_EDA_FIELDUAL_GET_MEDIUM       _IOWR(MEMORY_DEVICE, 30, int)

// --- IRQ management
#define IOCTL_EDA_IRQ_WAIT_FOR              _IOWR(MEMORY_DEVICE, 31, int)
#define IOCTL_EDA_IRQ_REARM                 _IOWR(MEMORY_DEVICE, 32, int)
#define IOCTL_EDA_IRQ_RESUME                _IOWR(MEMORY_DEVICE, 33, int)

// --- Statistics
#define IOCTL_EDA_RESET_STATS               _IOWR(MEMORY_DEVICE, 34, int)
#define IOCTL_EDA_GET_STATS                 _IOWR(MEMORY_DEVICE, 35, int)

// IOCTL FCA (FipCode Access) - related to FullFip IOSpace
// --- System Management
#define IOCTL_FCA_START         _IOWR(MEMORY_DEVICE, 50, int)
#define IOCTL_FCA_TST_WFIFO     _IOWR(MEMORY_DEVICE, 51, int)
#define IOCTL_FCA_TST_RFIFO     _IOWR(MEMORY_DEVICE, 52, int)
#define IOCTL_FCA_TST_IRQ       _IOWR(MEMORY_DEVICE, 53, int)
#define IOCTL_FCA_TST_REG       _IOWR(MEMORY_DEVICE, 54, int)
#define IOCTL_FCA_TST_TIMER     _IOWR(MEMORY_DEVICE, 55, int)
#define IOCTL_FCA_VALID         _IOWR(MEMORY_DEVICE, 56, int)
#define IOCTL_FCA_GET_MEDIUM    _IOWR(MEMORY_DEVICE, 57, int)
#define IOCTL_FCA_VALID_MEDIUM  _IOWR(MEMORY_DEVICE, 58, int)
#define IOCTL_FCA_NOISE_MANAGEMENT _IOWR(MEMORY_DEVICE, 59, int)
// --- Memory Access
#define IOCTL_FCA_W_PHY         _IOWR(MEMORY_DEVICE, 60, int)
#define IOCTL_FCA_R_PHY         _IOWR(MEMORY_DEVICE, 61, int)
#define IOCTL_FCA_W_WORD        _IOWR(MEMORY_DEVICE, 62, int)
#define IOCTL_FCA_R_WORD        _IOWR(MEMORY_DEVICE, 63, int)
#define IOCTL_FCA_W_DESC        _IOWR(MEMORY_DEVICE, 64, int)
// --- Variable Access
#define IOCTL_FCA_W_VAR         _IOWR(MEMORY_DEVICE, 65, int)
#define IOCTL_FCA_R_VAR         _IOWR(MEMORY_DEVICE, 66, int)
#define IOCTL_FCA_R_VAR_SVF     _IOWR(MEMORY_DEVICE, 67, int)
#define IOCTL_FCA_R_VAR_TIME    _IOWR(MEMORY_DEVICE, 68, int)
#define IOCTL_FCA_R_LPRESENT    _IOWR(MEMORY_DEVICE, 69, int)
#define IOCTL_FCA_SEND_APER     _IOWR(MEMORY_DEVICE, 70, int)
#define IOCTL_FCA_PURGE_APER    _IOWR(MEMORY_DEVICE, 71, int)
// --- Event Access
#define IOCTL_FCA_R_EVT         _IOWR(MEMORY_DEVICE, 72, int)
#define IOCTL_FCA_ACQ_EOC       _IOWR(MEMORY_DEVICE, 73, int)
// --- Message Access
#define IOCTL_FCA_W_MSG         _IOWR(MEMORY_DEVICE, 74, int)
#define IOCTL_FCA_R_MSG         _IOWR(MEMORY_DEVICE, 75, int)
#define IOCTL_FCA_PURGE_MSG     _IOWR(MEMORY_DEVICE, 76, int)
// --- Ba Access
#define IOCTL_FCA_START_BA      _IOWR(MEMORY_DEVICE, 77, int)
#define IOCTL_FCA_STOP_BA       _IOWR(MEMORY_DEVICE, 78, int)
#define IOCTL_FCA_CONTINUE_BA   _IOWR(MEMORY_DEVICE, 79, int)
#define IOCTL_FCA_CHANGE_MAC    _IOWR(MEMORY_DEVICE, 80, int)
#define IOCTL_FCA_SYNC_BA       _IOWR(MEMORY_DEVICE, 81, int)
#define IOCTL_FCA_CHANGE_PARAM  _IOWR(MEMORY_DEVICE, 82, int)

#elif defined(__QNX__)
typedef union _devctl_get_dev_info_msg {
    struct _EDA_DEV_INFO_OUTPUT o;
} devctl_get_dev_info_msg;

typedef union _devctl_list_internal_reg_msg {
    struct _EDA_LIST_INTERNAL_REGISTERS_OUTPUT o;
} devctl_list_internal_reg_msg;

typedef union _devctl_plx_write_reg_msg {
    struct _EDA_WRITE_REGISTER_INPUT i;
} devctl_plx_write_reg_msg;

typedef union _devctl_plx_read_reg_msg {
    __u32 i_relative_addr;
    __u32 o_value;
} devctl_plx_read_reg_msg;

typedef union _devctl_check_eeprom_msg {
    __u32 o_eeprom_state;
} devctl_check_eeprom_msg;

typedef union _devctl_read_eeprom_msg {
    struct _EDA_READ_EEPROM_INPUT i;
    struct _EDA_READ_EEPROM_OUTPUT o;
} devctl_read_eeprom_msg;

typedef union _devctl_write_eeprom_msg {
    struct _EDA_WRITE_EEPROM_INPUT i;
    __u8 o_written_bytes;
} devctl_write_eeprom_msg;

typedef union _devctl_get_rgb_led_msg {
    __u8 o_led_state;
} devctl_get_rgb_led_msg;

typedef union _devctl_set_rgb_led_msg {
    __u8 i_led_state;
} devctl_set_rgb_led_msg;

typedef union _devctl_get_product_number_msg {
     __u32 o_product_number;
} devctl_get_product_number_msg;

typedef union _devctl_set_product_number_msg {
    __u32 i_product_number;
    __u8 o_nb_bytes_written;
} devctl_set_product_number_msg;

typedef union _devctl_get_device_id_msg {
    __u16 o_device_id;
} devctl_get_device_id_msg;

typedef union _devctl_set_device_id_msg {
    __u16 i_device_id;
    __u8 o_nb_bytes_written;
} devctl_set_device_id_msg;

typedef union _devctl_get_card_conf_msg {
    __u32 o_card_conf;
} devctl_get_card_conf_msg;

typedef union _devctl_set_card_conf_msg {
    __u32 i_card_conf;
    __u8 o_nb_bytes_written;
} devctl_set_card_conf_msg;

typedef union _devctl_get_subsystem_vendor_id_msg {
    __u16 o_subsystem_vid;
} devctl_get_subsystem_vendor_id_msg;

typedef union _devctl_set_subsystem_vendor_id_msg {
    __u16 i_subsystem_vid;
    __u8 o_nb_bytes_written;
} devctl_set_subsystem_vendor_id_msg;

typedef union _devctl_get_subsystem_id_msg {
    __u16 o_subsystem_did;
} devctl_get_subsystem_id_msg;

typedef union _devctl_set_subsystem_id_msg {
    __u16 i_subsystem_did;
    __u8 o_nb_bytes_written;
} devctl_set_subsystem_id_msg;

typedef union _devctl_get_revision_id_msg {
    __u16 o_revision_did;
} devctl_get_revision_id_msg;

typedef union _devctl_set_revision_id_msg {
    __u16 i_revision_did;
    __u8 o_nb_bytes_written;
} devctl_set_revision_id_msg;

typedef union _devctl_get_monitoring_infos_msg {
    struct _EDA_GET_MONITORING_INFOS_OUTPUT o;
} devctl_get_monitoring_infos_msg;

typedef union _devctl_fieldual_valid_medium_msg {
    __u16 i_fieldual_cmd;
    __u8 o_medium_state;
} devctl_fieldual_valid_medium_msg;

typedef union _devctl_fieldual_get_medium_msg {
    __u8 o_medium_state;
} devctl_fieldual_get_medium_msg;

typedef union _devctl_irq_wait_for_msg {
    __u16 o_result;
} devctl_irq_wait_for_msg;

typedef union _devctl_get_stats_msg {
    struct _EDA_GET_STATS_OUTPUT o;
} devctl_get_stats_msg;

typedef union _devctl_start_msg {
    __u16 i_start_prog_addr;
    __u8 o_varstate;
} devctl_start_msg;

typedef union _devctl_tst_wfifo_msg {
    __u8 i_tab[8];
    __u8 o_ustate;
} devctl_tst_wfifo_msg;

typedef union _devctl_tst_rfifo_msg {
    __u16 o_tst_result;
} devctl_tst_rfifo_msg;

typedef union _devctl_tst_irq_msg {
    __u8 o_ustate;
} devctl_tst_irq_msg;

typedef union _devctl_tst_reg_msg {
    __u16 i_val;
    __u16 o_result;
} devctl_tst_reg_msg;

typedef union _devctl_tst_timer_msg {
    __u16 i_val;
    __u16 o_result;
} devctl_tst_timer_msg;

typedef union _devctl_valid_msg {
    struct _FIP_VALID_CIRCUIT_OUTPUT o;
} devctl_valid_msg;

typedef union _devctl_get_medium_msg {
    struct _FIP_GET_MEDIUM_OUTPUT o;
} devctl_get_medium_msg;

typedef union _devctl_valid_medium_msg {
    __u16 i_fieldual_cmd;
    __u8 o_varstate;
} devctl_valid_medium_msg;

typedef union _devctl_noise_management_msg {
    __u16 i_noise_detection; /* 1: true, 0: false */
    __u8 o_varstate;
} devctl_noise_management_msg;

typedef union _devctl_w_phy_msg {
    struct _FIP_WRITE_PHY_INPUT i;
    __u8 o_varstate;
} devctl_w_phy_msg;

typedef union _devctl_r_phy_msg {
    __u16 i_page_number;
    struct _FIP_READ_PHY_OUTPUT o;
} devctl_r_phy_msg;

typedef union _devctl_w_word_msg {
    struct _FIP_WRITE_WORD_INPUT i;
    __u8 o_varstate;
} devctl_w_word_msg;

typedef union _devctl_r_word_msg {
    __u32 i_address;
    struct _FIP_READ_WORD_OUTPUT o;
} devctl_r_word_msg;

typedef union _devctl_w_desc_msg {
    struct _FIP_WRITE_DESC_INPUT i;
    __u8 o_varstate;
} devctl_w_desc_msg;

typedef union _devctl_w_var_msg {
    struct _FIP_WRITE_VAR_INPUT i;
    __u8 o_varstate;
} devctl_w_var_msg;

typedef union _devctl_r_var_msg {
    __u16 i_access_key;
    struct _FIP_READ_VAR_OUTPUT o;
} devctl_r_var_msg;

typedef union _devctl_r_var_svf_msg {
    __u16 i_access_key;
    struct _FIP_READ_VAR_OUTPUT o;
} devctl_r_var_svf_msg;

typedef union _devctl_r_var_time_msg {
    __u16 i_access_key;
    struct _FIP_READ_VAR_TIME_OUTPUT o;
} devctl_r_var_time_msg;

typedef union _devctl_r_lpresent_msg {
    struct _FIP_READ_LSTPRESENT_OUTPUT o;
} devctl_r_lpresent_msg;

typedef union _devctl_send_aper_msg {
    struct _FIP_SEND_APER_INPUT i;
    __u8 o_varstate;
} devctl_send_aper_msg;

typedef union _devctl_purge_aper_msg {
    __u8 i_fifo_type; /* 1: urgent, 0: normal */
    __u8 o_varstate;
} devctl_purge_aper_msg;

typedef union _devctl_r_evt_msg {
    struct _FIP_READ_EVT_OUTPUT o;
} devctl_r_evt_msg;

typedef union _devctl_acq_eoc_msg {
    __u16 i_acq_eoc; /* ack on: 1: high level, 0: low level */
    __u8 o_varstate;
} devctl_acq_eoc_msg;

typedef union _devctl_w_msg_msg {
    struct _FIP_WRITE_MSG_INPUT i;
    __u8 o_varstate;
} devctl_w_msg_msg;

typedef union _devctl_r_msg_msg {
    struct _FIP_READ_MSG_OUTPUT o;
} devctl_r_msg_msg;

typedef union _devctl_purge_msg_msg {
    __u16 i_channel_num;
    struct _FIP_PURGE_MSG_OUTPUT o;
} devctl_purge_msg_msg;

typedef union _devctl_start_ba_msg {
    __u16 i_macrocycle_addr;
    __u8 o_varstate;
} devctl_start_ba_msg;

typedef union _devctl_stop_ba_msg {
    __u8 o_varstate;
} devctl_stop_ba_msg;

typedef union _devctl_continue_ba_msg {
    __u16 i_ba_continue_addr;
    __u8 o_varstate;
} devctl_continue_ba_msg;

typedef union _devctl_change_mac_msg {
    __u16 i_macrocycle_addr;
    __u8 o_varstate;
} devctl_change_mac_msg;

typedef union _devctl_sync_ba_msg {
    __u8 o_varstate;
} devctl_sync_ba_msg;

typedef union _devctl_change_param_msg {
    struct _FIP_CHANGE_PARAM_INPUT i;
    __u8 o_varstate;
} devctl_change_param_msg;

typedef union _devctl_msg
{
    devctl_get_dev_info_msg get_dev_info_msg;
    devctl_list_internal_reg_msg list_internal_reg_msg;
    devctl_plx_write_reg_msg plx_write_reg_msg;
    devctl_plx_read_reg_msg plx_read_reg_msg;
    devctl_check_eeprom_msg check_eeprom_msg;
    devctl_read_eeprom_msg read_eeprom_msg;
    devctl_write_eeprom_msg write_eeprom_msg;
    devctl_get_rgb_led_msg get_rgb_led_msg;
    devctl_set_rgb_led_msg set_rgb_led_msg;
    devctl_get_product_number_msg get_product_number_msg;
    devctl_set_product_number_msg set_product_number_msg;
    devctl_get_device_id_msg get_device_id_msg;
    devctl_set_device_id_msg set_device_id_msg;
    devctl_get_card_conf_msg get_card_conf_msg;
    devctl_set_card_conf_msg set_card_conf_msg;
    devctl_get_subsystem_vendor_id_msg get_subsystem_vendor_id_msg;
    devctl_set_subsystem_vendor_id_msg set_subsystem_vendor_id_msg;
    devctl_get_subsystem_id_msg get_subsystem_id_msg;
    devctl_set_subsystem_id_msg set_subsystem_id_msg;
    devctl_get_revision_id_msg get_revision_id_msg;
    devctl_set_revision_id_msg set_revision_id_msg;
    devctl_get_monitoring_infos_msg get_monitoring_infos_msg; 
    devctl_fieldual_valid_medium_msg fieldual_valid_medium_msg;
    devctl_fieldual_get_medium_msg fieldual_get_medium_msg;
    devctl_irq_wait_for_msg irq_wait_for_msg;
    devctl_get_stats_msg get_stats_msg;
    devctl_start_msg start_msg;
    devctl_tst_wfifo_msg tst_wfifo_msg;
    devctl_tst_rfifo_msg tst_rfifo_msg;
    devctl_tst_irq_msg tst_irq_msg;
    devctl_tst_reg_msg tst_reg_msg;
    devctl_tst_timer_msg tst_timer_msg;
    devctl_valid_msg valid_msg;
    devctl_get_medium_msg get_medium_msg;
    devctl_valid_medium_msg valid_medium_msg;
    devctl_noise_management_msg noise_management_msg;
    devctl_w_phy_msg w_phy_msg;
    devctl_r_phy_msg r_phy_msg;
    devctl_w_word_msg w_word_msg;
    devctl_r_word_msg r_word_msg;
    devctl_w_desc_msg w_desc_msg;
    devctl_w_var_msg w_var_msg;
    devctl_r_var_msg r_var_msg;
    devctl_r_var_svf_msg r_var_svf_msg;
    devctl_r_var_time_msg r_var_time_msg;
    devctl_r_lpresent_msg r_lpresent_msg;
    devctl_send_aper_msg send_aper_msg;
    devctl_purge_aper_msg purge_aper_msg;
    devctl_r_evt_msg r_evt_msg;
    devctl_acq_eoc_msg acq_eoc_msg;
    devctl_w_msg_msg w_msg_msg;
    devctl_r_msg_msg r_msg_msg;
    devctl_purge_msg_msg purge_msg_msg;
    devctl_start_ba_msg start_ba_msg;
    devctl_stop_ba_msg stop_ba_msg;
    devctl_continue_ba_msg continue_ba_msg;
    devctl_change_mac_msg change_mac_msg;
    devctl_sync_ba_msg sync_ba_msg;
    devctl_change_param_msg change_param_msg;
    /* ... other devctl types you can receive */
} devctl_msg, *devctl_msg_ptr;

#define CMD_CODE 1072014
// IOCTL EDA (ExoDevice Access)
// --- Management
#define IOCTL_EDA_GET_DEV_INFO              __DIOF(_DCMD_ALL, CMD_CODE + 0, union _devctl_get_dev_info_msg) // O
#define IOCTL_EDA_LIST_INTERNAL_REGISTERS   __DIOF(_DCMD_ALL, CMD_CODE + 1, union _devctl_list_internal_reg_msg) // O
#define IOCTL_EDA_RELOAD_INTERNAL_REGISTERS __DION(_DCMD_ALL, CMD_CODE + 2) // none
#define IOCTL_EDA_PLX_WRITE_REGISTER        __DIOT(_DCMD_ALL, CMD_CODE + 4, union _devctl_plx_write_reg_msg) // I
#define IOCTL_EDA_PLX_READ_REGISTER         __DIOTF(_DCMD_ALL, CMD_CODE + 5, union _devctl_plx_read_reg_msg) // I + O
// --- EEPROM Management
#define IOCTL_EDA_CHECK_EEPROM              __DIOF(_DCMD_ALL, CMD_CODE + 6, union _devctl_check_eeprom_msg) // O
#define IOCTL_EDA_READ_EEPROM               __DIOTF(_DCMD_ALL, CMD_CODE + 7, union _devctl_read_eeprom_msg) // I + O
#define IOCTL_EDA_WRITE_EEPROM              __DIOTF(_DCMD_ALL, CMD_CODE + 8, union _devctl_write_eeprom_msg) // I + O
// --- Useful I/O access
#define IOCTL_EDA_GET_RGB_LED               __DIOF(_DCMD_ALL, CMD_CODE + 9, union _devctl_get_rgb_led_msg) // O
#define IOCTL_EDA_SET_RGB_LED               __DIOT(_DCMD_ALL, CMD_CODE + 10, union _devctl_set_rgb_led_msg) // I
// --- Useful information access
#define IOCTL_EDA_GET_PRODUCT_NUMBER        __DIOF(_DCMD_ALL, CMD_CODE + 11, union _devctl_get_product_number_msg) // O
#define IOCTL_EDA_SET_PRODUCT_NUMBER        __DIOTF(_DCMD_ALL, CMD_CODE + 12, union _devctl_set_product_number_msg) // I + O
#define IOCTL_EDA_GET_DEVICE_ID             __DIOF(_DCMD_ALL, CMD_CODE + 13, union _devctl_get_device_id_msg) // O
#define IOCTL_EDA_SET_DEVICE_ID             __DIOTF(_DCMD_ALL, CMD_CODE + 14, union _devctl_set_device_id_msg) // I + O
#define IOCTL_EDA_GET_CARD_CONF             __DIOF(_DCMD_ALL, CMD_CODE + 15, union _devctl_get_card_conf_msg) // O
#define IOCTL_EDA_SET_CARD_CONF             __DIOTF(_DCMD_ALL, CMD_CODE + 16, union _devctl_set_card_conf_msg) // I + O
#define IOCTL_EDA_GET_SUBSYSTEM_VENDOR_ID   __DIOF(_DCMD_ALL, CMD_CODE + 17, union _devctl_get_subsystem_vendor_id_msg) // O
#define IOCTL_EDA_SET_SUBSYSTEM_VENDOR_ID   __DIOTF(_DCMD_ALL, CMD_CODE + 18, union _devctl_set_subsystem_vendor_id_msg) // I + O
#define IOCTL_EDA_GET_SUBSYSTEM_ID          __DIOF(_DCMD_ALL, CMD_CODE + 19, union _devctl_get_subsystem_id_msg) // O
#define IOCTL_EDA_SET_SUBSYSTEM_ID          __DIOTF(_DCMD_ALL, CMD_CODE + 20, union _devctl_set_subsystem_id_msg) // I + O
#define IOCTL_EDA_GET_REVISION_ID           __DIOF(_DCMD_ALL, CMD_CODE + 21, union _devctl_get_revision_id_msg) // O
#define IOCTL_EDA_SET_REVISION_ID           __DIOTF(_DCMD_ALL, CMD_CODE + 22, union _devctl_set_revision_id_msg) // I + O
#define IOCTL_EDA_GET_MONITORING_INFOS      __DIOF(_DCMD_ALL, CMD_CODE + 64, union _devctl_get_monitoring_infos_msg) // O

// --- FullFip access
#define IOCTL_EDA_FULLFIP_RESET             __DION(_DCMD_ALL, CMD_CODE + 23) // none
// --- FielDual / MAX3000A access
#define IOCTL_EDA_FIELDUAL_VALID_MEDIUM     __DIOTF(_DCMD_ALL, CMD_CODE + 24, union _devctl_fieldual_valid_medium_msg) // I + O
#define IOCTL_EDA_FIELDUAL_GET_MEDIUM       __DIOF(_DCMD_ALL, CMD_CODE + 25, union _devctl_fieldual_get_medium_msg) // O
// --- IRQ management
#define IOCTL_EDA_IRQ_WAIT_FOR              __DIOF(_DCMD_ALL, CMD_CODE + 26, union _devctl_irq_wait_for_msg) // O
#define IOCTL_EDA_IRQ_REARM                 __DION(_DCMD_ALL, CMD_CODE + 27) // none
#define IOCTL_EDA_IRQ_RESUME                __DION(_DCMD_ALL, CMD_CODE + 28) // none
// --- Statistics
#define IOCTL_EDA_RESET_STATS               __DION(_DCMD_ALL, CMD_CODE + 29) // none
#define IOCTL_EDA_GET_STATS                 __DIOF(_DCMD_ALL, CMD_CODE + 30, union _devctl_get_stats_msg) // O
// IOCTL FCA (FipCode Access) - related to FullFip IOSpace
// --- System Management
#define IOCTL_FCA_START                     __DIOTF(_DCMD_ALL, CMD_CODE + 31, union _devctl_start_msg) // I + O
#define IOCTL_FCA_TST_WFIFO                 __DIOTF(_DCMD_ALL, CMD_CODE + 32, union _devctl_tst_wfifo_msg) // I + O
#define IOCTL_FCA_TST_RFIFO                 __DIOF(_DCMD_ALL, CMD_CODE + 33, union _devctl_tst_rfifo_msg) // O
#define IOCTL_FCA_TST_IRQ                   __DIOF(_DCMD_ALL, CMD_CODE + 34, union _devctl_tst_irq_msg) // O
#define IOCTL_FCA_TST_REG                   __DIOTF(_DCMD_ALL, CMD_CODE + 35, union _devctl_tst_reg_msg) // I + O
#define IOCTL_FCA_TST_TIMER                 __DIOTF(_DCMD_ALL, CMD_CODE + 36, union _devctl_tst_timer_msg) // I + O
#define IOCTL_FCA_VALID                     __DIOF(_DCMD_ALL, CMD_CODE + 37, union _devctl_valid_msg) // O
#define IOCTL_FCA_GET_MEDIUM                __DIOF(_DCMD_ALL, CMD_CODE + 38, union _devctl_get_medium_msg) // O
#define IOCTL_FCA_VALID_MEDIUM              __DIOTF(_DCMD_ALL, CMD_CODE + 39, union _devctl_valid_medium_msg) // I + O
#define IOCTL_FCA_NOISE_MANAGEMENT          __DIOTF(_DCMD_ALL, CMD_CODE + 40, union _devctl_noise_management_msg) // I + O
// --- Memory Access
#define IOCTL_FCA_W_PHY                     __DIOTF(_DCMD_ALL, CMD_CODE + 41, union _devctl_w_phy_msg) // I + O
#define IOCTL_FCA_R_PHY                     __DIOTF(_DCMD_ALL, CMD_CODE + 42, union _devctl_r_phy_msg) // I + O
#define IOCTL_FCA_W_WORD                    __DIOTF(_DCMD_ALL, CMD_CODE + 43, union _devctl_w_word_msg) // I + O
#define IOCTL_FCA_R_WORD                    __DIOTF(_DCMD_ALL, CMD_CODE + 44, union _devctl_r_word_msg) // I + O
#define IOCTL_FCA_W_DESC                    __DIOTF(_DCMD_ALL, CMD_CODE + 45, union _devctl_w_desc_msg) // I + O
// --- Variable Access
#define IOCTL_FCA_W_VAR                     __DIOTF(_DCMD_ALL, CMD_CODE + 46, union _devctl_w_var_msg) // I + O
#define IOCTL_FCA_R_VAR                     __DIOTF(_DCMD_ALL, CMD_CODE + 47, union _devctl_r_var_msg) // I + O
#define IOCTL_FCA_R_VAR_SVF                 __DIOTF(_DCMD_ALL, CMD_CODE + 48, union _devctl_r_var_svf_msg) // I + O
#define IOCTL_FCA_R_VAR_TIME                __DIOTF(_DCMD_ALL, CMD_CODE + 49, union _devctl_r_var_time_msg) // I + O
#define IOCTL_FCA_R_LPRESENT                __DIOF(_DCMD_ALL, CMD_CODE + 50, union _devctl_r_lpresent_msg) // O
#define IOCTL_FCA_SEND_APER                 __DIOTF(_DCMD_ALL, CMD_CODE + 51, union _devctl_send_aper_msg) // I + O
#define IOCTL_FCA_PURGE_APER                __DIOTF(_DCMD_ALL, CMD_CODE + 52, union _devctl_purge_aper_msg) // I + O
// --- Event Access
#define IOCTL_FCA_R_EVT                     __DIOF(_DCMD_ALL, CMD_CODE + 53, union _devctl_r_evt_msg) // O
#define IOCTL_FCA_ACQ_EOC                   __DIOTF(_DCMD_ALL, CMD_CODE + 54, union _devctl_acq_eoc_msg) // I + O
// --- Message Access
#define IOCTL_FCA_W_MSG                     __DIOTF(_DCMD_ALL, CMD_CODE + 55, union _devctl_w_msg_msg) // I + O
#define IOCTL_FCA_R_MSG                     __DIOF(_DCMD_ALL, CMD_CODE + 56, union _devctl_r_msg_msg) // O
#define IOCTL_FCA_PURGE_MSG                 __DIOTF(_DCMD_ALL, CMD_CODE + 57, union _devctl_purge_msg_msg) // I + O
// --- Ba Access
#define IOCTL_FCA_START_BA                  __DIOTF(_DCMD_ALL, CMD_CODE + 58, union _devctl_start_ba_msg) // I + O
#define IOCTL_FCA_STOP_BA                   __DIOF(_DCMD_ALL, CMD_CODE + 59, union _devctl_stop_ba_msg) // O
#define IOCTL_FCA_CONTINUE_BA               __DIOTF(_DCMD_ALL, CMD_CODE + 60, union _devctl_continue_ba_msg) // I + O
#define IOCTL_FCA_CHANGE_MAC                __DIOTF(_DCMD_ALL, CMD_CODE + 61, union _devctl_change_mac_msg) // I + O
#define IOCTL_FCA_SYNC_BA                   __DIOF(_DCMD_ALL, CMD_CODE + 62, union _devctl_sync_ba_msg) // O
#define IOCTL_FCA_CHANGE_PARAM              __DIOTF(_DCMD_ALL, CMD_CODE + 63, union _devctl_change_param_msg) // I + O
#endif

//***************************************************************************************
//                              IProtocol Definition
//                              MobiFIP device (USB)
//***************************************************************************************
// Note: The device version number contained in CARDCONF determines the firmware 
// compatibility between FipCode 5 (FipEngine) and FipCode6 (FipCore)
// FipCode 5: version between 0.0 and 1.15
// FipCode 6: version from 2.0

// DST - Destination
#define DST_MOBIFIP 1

// ID - Identifier
// read_by_id / write_by_id
#define ID_RGB              (unsigned char) 0x03 // R/W
#define ID_SN               (unsigned char) 0x04 // R/W
#define ID_CARDCONF         (unsigned char) 0x05 // R/W
#define ID_FIRMWARE_VERSION (unsigned char) 0x06 // R/W
// exec_by_id
#define ID_MEDIUM_ACCESS    (unsigned char) 0x50
#define ID_FULLFIP_RAZ_MEM  (unsigned char) 0xcc
//  -> ID FCA (FipCode Access)
//  The IProtocol ID is written to the FipCode user commands:
// CMDE_R_PHY, CMDE_R_WORD, CMDE_W_PHY, CMDE_W_WORD, CMDE_W_DESC, CMDE_R_VAR,
// CMDE_R_VAR_SVF, CMDE_R_VAR_TIME, CMDE_R_LPRESENT, CMDE_W_VAR, CMDE_R_EVT, CMDE_ACQ_EOC,
// CMDE_R_MSG, CMDE_PURGE_MSG, CMDE_W_MSG, CMDE_SEND_APU, CMDE_SEND_APN, CMDE_PURGE_APER,
// CMDE_START_BA, CMDE_STOP_BA, CMDE_CHANGE_MAC, CMDE_CONTINUE_BA, CMDE_SYNC_BA,
// CMDE_CHANGE_PARAM, CMDE_GET_MEDIUM, CMDE_VALID_MEDIUM, CMDE_GES_BRUIT, CMDE_TST_RFIFO,
// CMDE_TST_IRQ, CMDE_TST_REG, CMDE_TST_TIME, CMDE_TST_WFIFO, CMDE_START, CMDE_VALID,
// CMDE_CLOSE


//-- The IDs below will be removed
//  -> ID EDA (ExoDevice Access)
/*#define CMDE_CHECK_EEPROM           (unsigned char) 0x0A
#define CMDE_READ_EEPROM            (unsigned char) 0x0B
#define CMDE_WRITE_EEPROM           (unsigned char) 0x0C
#define CMDE_GET_PRODUCT_NUMBER     (unsigned char) 0x0D
#define CMDE_SET_PRODUCT_NUMBER     (unsigned char) 0x0E
#define CMDE_GET_FIP_SPEED          (unsigned char) 0x0F
#define CMDE_SET_FIP_SPEED          (unsigned char) 0x1A
#define CMDE_GET_FIP_IMPEDANCE      (unsigned char) 0x1B
#define CMDE_SET_FIP_IMPEDANCE      (unsigned char) 0x1C
#define CMDE_GET_VERSION            (unsigned char) 0x1D
#define CMDE_SET_VERSION            (unsigned char) 0x1E

#define CMDE_GET_DEVICE_ID          (unsigned char) 0x1F
#define CMDE_GET_SUBSYSTEM_VENDOR_ID (unsigned char) 0x2A
#define CMDE_GET_SUBSYSTEM_ID       (unsigned char) 0x2B
#define CMDE_GET_REVISION_ID        (unsigned char) 0x2C

#define CMDE_FULLFIP_RESET          (unsigned char) 0x2D

#define CMDE_FIELDUAL_GET_MEDIUM    (unsigned char) 0x2E

#define CMDE_GET_RGB_LED            (unsigned char) 0x2F

#define CMDE_SET_RGB_LED            (unsigned char) 0x3A*/

#endif
