/*
 * Copyright (C) 2014-2019 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __FA_DRV_H__
#define __FA_DRV_H__

#include <linux/cdev.h>
#include <linux/semaphore.h>
#include <linux/ktime.h>

#define PCI_MAJOR   0   /* dynamic major by default */
#define PCI_DEVS    4

#define STATUS_SUCCESS                  0
#define STATUS_TIMEOUT                  0x01
#define STATUS_DEVICE_BUSY              0x02
#define STATUS_UNSUCCESSFUL             0x03
#define STATUS_INVALID_PARAMETER        0x04
#define STATUS_INVALID_DEVICE_STATE     0x05

// PLX UserIO, Serial EEPROM, and
extern int pci_major ;
extern int pci_devs ;

typedef enum {
    MY_BOARD = 0,
} board_t;

typedef struct _INFO_IO_SPACE
{
    bool      bSpaceMapped;
    uintptr_t nSpaceRange;
    uintptr_t physicalAddress;
    void*     pSpaceBase;
} _INFO_IO_SPACE, *_PINFO_IO_SPACE;

typedef struct fip_stat {
    /*** EOC */
    /* number of macrocycles - linked with fip synchro var (0x9003) */
    u64 cycle_count;
    /* macrocycle time - nanoscond resolution */
    u64 cycle_time;

    /*** IRQ */
    /* number of variables sent with a network event flag sets to "permanent" */
    u64 var_p_sent_count;
    /* number of variables sent with a network event flag sets to "temporary" */
    u64 var_t_sent_count;
    /* number of variables received with a network event flag sets to "permanent" */
    u64 var_p_received_count;
    /* number of variables received with a network event flag sets to "temporary" */
    u64 var_t_received_count;
    /* number of message received */
    u64 msg_received_count;
    /* number of message sent */
    u64 msg_sent_count;
    /* number of "urgent" requests for aperiodic variable list sent */
    u64 apu_list_sent_count;
    /* number of "normal" requests for aperiodic variable list sent */
    u64 apn_list_sent_count;
    /* counter of bus arbiter (re)activation */
    u64 ba_activity_count;
    /* counter of stopping of bus arbiter following timeout */
    u64 ba_stop_on_timeout_count;
    /* counter of stopping of bus arbiter following network anomalies */
    u64 ba_stop_on_anomalies_count;
    /* counter of stopping of bus arbiter following user request */
    u64 ba_stop_by_user_count;
    /* counter of return of bus arbiter to idle status */
    u64 ba_idle_count;
    /* counter of bus arbiter suspend */
    u64 ba_suspend_count;
} fip_stat;

struct _reg {
    /* fullfip */
    // r
    u8 ustate;
    u8 uflags;
    u8 var_state;
    u8 size16_l;
    u8 size16_h;
    // w
    u8 ucom;
    u8 uswitch;
    u8 key_h;
    u8 key_l;
    u8 time_h;
    u8 time_l;
    // r/w
    u8 file;
    /* fieldual */
    // r
    u8 ch_status;
    u8 clr_err;
    u8 reset_ch_1;
    u8 reset_ch_2;
    // w
    u8 loopback_sel;
    u8 enable_ch_1;
    u8 enable_ch_2;
    u8 enable_ch_1_2;
};


struct pci_priv_device {
    struct list_head    link;       /* Double linked list */
    struct pci_dev      *pci_dev;   /* PCI device */
    struct device       *dev;
    int                 is_open;    /* 0: free - 1: busy */
    board_t             type;       /* Board type */
    u32                 minor;      /* Minor number */
    u32                 CardConf;
    u8                  LedsState;
    _INFO_IO_SPACE      IoSpacePciPlx;
    _INFO_IO_SPACE      IoSpacePciFullFip2;
    _INFO_IO_SPACE      IoSpacePciFieldual;
    _INFO_IO_SPACE      IoSpacePciRam;
    uintptr_t           IoPortPciPlxAddress;
    uintptr_t           IoPortPciPlxSize;
    struct semaphore    sem;        /* Mutual exclusion */
    struct cdev         cdev;
    int                 int_flag;
    u8                  linti1;
    u8                  linti2;
    u32                 irq_handled_counter;
    u32                 irq_none_handled_counter;
    wait_queue_head_t   int_mutex;
    struct spinlock     lock;
    struct fip_stat     stat;
    ktime_t             prectime;

    /* irq line successfully requested by this driver, -1 otherwise */
    int                 irq_line;

    int                 nb_fullfip;
    int                 nb_fullfip_ram;
    int                 nb_fieldual;
    u8                  reg_offset_factor; 
                                    /* Hardware register offset for
                                     * plx/igloo2 <-> fullfip communication
                                     * plx: 32-bits alignment (offset = 4)
                                     * igloo2: 64-bits alignment (offset = 8)
                                     */
    struct _reg         reg;        /* Structure containing the fullfip and
                                     * fieldual registers
                                     */
};

extern struct file_operations pcifip_fops;

#endif
